from appdirs import user_data_dir
import configparser
from os import makedirs
import os.path

APP_NAME = "pnplabs"
AUTHOR_NAME = "pnplabs"

USER_CONFIG_DIR_PATH = user_data_dir(APP_NAME, AUTHOR_NAME)
USER_CONFIG_PATH = os.path.join(USER_CONFIG_DIR_PATH, "config.ini")


def get_config():
    config = configparser.ConfigParser()
    if os.path.exists(USER_CONFIG_PATH):
        config.read(USER_CONFIG_PATH)
    return config


def get_config_as_str():
    if os.path.exists(USER_CONFIG_PATH):
        with open(USER_CONFIG_PATH) as fobj:
            return fobj.read()


def dump_config(config):
    os.makedirs(USER_CONFIG_DIR_PATH, exist_ok=True)
    with open(USER_CONFIG_PATH, 'w') as configfile:
        config.write(configfile)



from pnplabs.utils.firestore import get_uid


def get_commands(db_client, api_key):
    teams_ids = [x.id for x in db_client.collection("teams").where("members", "array_contains", get_uid(api_key)).stream()]
    commands = [x.to_dict() for x in db_client.collection("commands").where(u"audience", u'in', [get_uid(api_key)] + teams_ids).stream()]
    return commands



from pnplabs.functions.connect import get_api_key
from pnplabs.utils.firestore import get_db_client
from pnplabs.utils.commands import get_commands
import subprocess


def execute(*args, **kwargs):
    # print(f"Executing dynamic command with {args}")

    dynamic_function_name = args[0]

    api_key = get_api_key()
    if not api_key:
        print("Cannot run dynamic functions before connection. Please connect first.")
        return

    db_client = get_db_client(api_key)
    commands = get_commands(db_client, api_key)
    for command in commands:
        if command["name"] == dynamic_function_name:
            #print("found cloud command!")
            result = subprocess.run(list(command["command"].split(' ')), stdout=subprocess.PIPE)
            print(f"{result.stdout.decode('utf-8')}", end='')
            return
    print("Command not found!")



from pnplabs.utils.firestore import get_db_client
from pnplabs.utils.local_config import get_config, dump_config
from pnplabs.utils.add_py_scripts_to_path import windows_runner

def get_api_key():
    config = get_config()
    return config['Connect']['api_key']


def execute(*args, **kwargs):
    # print(f"Executing connect command with {args}")
    api_key = args[0]

    config = get_config()
    config['Connect'] = {
        'api_key': api_key,
    }
    dump_config(config)

    # print(f"API key is {api_key}, getting client...")

    # Check the connection (without actually doing anything

    db_client = None
    try:
        db_client = get_db_client(api_key)
    except Exception as e:
        print("Connection failed")
    print(f"Connected!")



import json
import requests

from requests.exceptions import HTTPError
from google.oauth2.credentials import Credentials
from google.cloud.firestore import Client

FIREBASE_REST_API = "https://identitytoolkit.googleapis.com/v1/accounts"
FB_API_KEY = "AIzaSyA59_DoB7koBFkXDrwtO_orgJ3GDkZyO8M"
PNP_API_KEY = "06fba83f-0a08-4feb-b03c-8d642569325e"


def get_token_and_user_id_with_pnp_api_key(api_key):
    request_url = "https://us-central1-pnplabs-be.cloudfunctions.net/tokener"
    headers = {"content-type": "application/json; charset=UTF-8"}
    data = json.dumps({"apiKey": api_key})

    resp = requests.post(request_url, headers=headers, data=data)
    # Check for errors
    try:
        resp.raise_for_status()
    except HTTPError as e:
        raise HTTPError(e, resp.text)

    return resp.json()


def sign_in_with_token(token, api_key=FB_API_KEY):
    request_url = "%s:signInWithCustomToken?key=%s" % (FIREBASE_REST_API, api_key)
    headers = {"content-type": "application/json; charset=UTF-8"}
    data = json.dumps({"token": token, "returnSecureToken": True})

    resp = requests.post(request_url, headers=headers, data=data)
    # Check for errors
    try:
        resp.raise_for_status()
    except HTTPError as e:
        raise HTTPError(e, resp.text)

    return resp.json()


def refresh_token(refresh_token, api_key=FB_API_KEY):
    refresh_request_url = f"https://securetoken.googleapis.com/v1/token?key={api_key}"
    headers = {"content-type": "application/json; charset=UTF-8"}
    data = json.dumps({"refresh_token": refresh_token, "grant_type": "refresh_token"})

    resp = requests.post(refresh_request_url, headers=headers, data=data)
    # Check for errors
    try:
        resp.raise_for_status()
    except HTTPError as e:
        raise HTTPError(e, resp.text)

    return resp.json()


def get_uid(api_key):
    firebase_token_and_user_id = get_token_and_user_id_with_pnp_api_key(api_key)
    firebase_user_id = firebase_token_and_user_id["userId"]
    return firebase_user_id


def get_db_client(api_key):
    # print(f"pnplabs api key: {api_key}")

    firebase_token_and_user_id = get_token_and_user_id_with_pnp_api_key(api_key)
    firebase_token = firebase_token_and_user_id["token"]
    firebase_user_id = firebase_token_and_user_id["userId"]
    # print(f"Firebase token: {firebase_token}")
    # print(f"Firebase user id: {firebase_user_id}")

    firebase_sign_in_response = sign_in_with_token(firebase_token)
    firebase_oauth2_token = firebase_sign_in_response['idToken']

    # Use google.oauth2.credentials and the response object to create the correct user credentials
    google_firestore_credentials = Credentials(
            token=firebase_oauth2_token,
            token_uri="https://oauth2.googleapis.com/token",
            client_id="105641275062017409334",
    )

    db_client = Client(
        project="pnplabs-be",
        credentials=google_firestore_credentials
    )

    return db_client